@@ -49,15 +49,14 @@ from liblogger import log
 
 import shlex
 
-if gpodder.interface == gpodder.MAEMO:
-    import osso
+_ = gpodder.gettext
+
 
 class gPodderLib(object):
     def __init__( self):
         log('Creating gPodderLib()', sender=self)
         gpodder_dir = os.path.expanduser(os.path.join('~', '.config', 'gpodder'))
         if gpodder.interface == gpodder.MAEMO:
-            self.osso_c = osso.Context('gpodder_osso_sender', '1.0', False)
             old_dir = '/media/mmc2/gpodder/'
             if os.path.exists(os.path.join(old_dir, 'channels.opml')) and not os.path.exists(os.path.join(gpodder_dir, 'channels.opml')):
                 # migrate from old (0.13.0 and earlier) gpodder maemo versions
@@ -356,17 +355,6 @@ class gPodderLib(object):
             assert filename is not None
         db.mark_episode(episode.url, is_played=True)
 
-        if gpodder.interface == gpodder.MAEMO and not self.config.maemo_allow_custom_player:
-            # Use the built-in Nokia Mediaplayer here
-            filename = filename.encode('utf-8')
-            osso_rpc = osso.Rpc(self.osso_c)
-            service = 'com.nokia.mediaplayer'
-            path = '/com/nokia/mediaplayer'
-            if not '://' in filename:
-                filename = 'file://' + filename
-            osso_rpc.rpc_run(service, path, service, 'mime_open', (filename,))
-            return (True, service)
-
         # Determine the file type and set the player accordingly.  
         file_type = episode.file_type()
 
@@ -389,36 +377,6 @@ class gPodderLib(object):
             return ( False, command_line[0] )
         return ( True, command_line[0] )
 
-    def ext_command_thread(self, notification, command_line):
-        """
-        This is the function that will be called in a separate
-        thread that will call an external command (specified by
-        command_line). In case of problem (i.e. the command has
-        not been found or there has been another error), we will
-        call the notification function with two arguments - the
-        first being the error message and the second being the
-        title to be used for the error message.
-        """
- 
-        log("(ExtCommandThread) Excuting command Line [%s]", command_line)
-    
-        p = subprocess.Popen(command_line, shell=True, stdout=sys.stdout, stderr=sys.stderr)
-        result = p.wait()
- 
-        if result == 127:
-            title = _('User command not found')
-            message = _('The user command [%s] was not found.\nPlease check your user command settings in the preferences dialog.' % command_line)
-            notification(message, title)
-        elif result == 126:
-            title = _('User command permission denied')
-            message = _('Permission denied when trying to execute user command [%s].\nPlease check that you have permissions to execute this command.' % command_line)
-            notification(message, title)
-        elif result > 0 :
-            title = _('User command returned an error')
-            message = _('The user command [%s] returned an error code of [%d]' % (command_line,result))
-            notification(message, title)
-
-        log("(ExtCommandThread) Finished command line [%s] result [%d]",command_line,result)
 
 
 class HistoryStore( types.ListType):
