@@ -61,16 +61,19 @@ import feedparser
 
 from xml.sax import saxutils
 
+_ = gpodder.gettext
 
 if gpodder.interface == gpodder.MAEMO:
     ICON_AUDIO_FILE = 'gnome-mime-audio-mp3'
     ICON_VIDEO_FILE = 'gnome-mime-video-mp4'
+    ICON_GENERIC_FILE = 'text-x-generic'
     ICON_DOWNLOADING = 'qgn_toolb_messagin_moveto'
     ICON_DELETED = 'qgn_toolb_gene_deletebutton'
     ICON_NEW = 'qgn_list_gene_favor'
 else:
     ICON_AUDIO_FILE = 'audio-x-generic'
     ICON_VIDEO_FILE = 'video-x-generic'
+    ICON_GENERIC_FILE = 'text-x-generic'
     ICON_DOWNLOADING = gtk.STOCK_GO_DOWN
     ICON_DELETED = gtk.STOCK_DELETE
     ICON_NEW = gtk.STOCK_ABOUT
@@ -78,9 +81,35 @@ else:
 
 class HTTPAuthError(Exception): pass
 
-class podcastChannel(object):
+
+class PodcastModelObject(object):
+    """
+    A generic base class for our podcast model providing common helper
+    and utility functions.
+    """
+
+    @classmethod
+    def create_from_dict(cls, d, *args):
+        """
+        Create a new object, passing "args" to the constructor
+        and then updating the object with the values from "d".
+        """
+        o = cls(*args)
+        o.update_from_dict(d)
+        return o
+
+    def update_from_dict(self, d):
+        """
+        Updates the attributes of this object with values from the
+        dictionary "d" by using the keys found in "d".
+        """
+        for k in d:
+            if hasattr(self, k):
+                setattr(self, k, d[k])
+
+
+class PodcastChannel(PodcastModelObject):
     """holds data for a complete channel"""
-    SETTINGS = ('sync_to_devices', 'device_playlist_name','override_title','username','password')
     MAX_FOLDERNAME_LENGTH = 150
     icon_cache = {}
 
@@ -91,11 +120,11 @@ class podcastChannel(object):
         if isinstance(url, unicode):
             url = url.encode('utf-8')
 
-        tmp = db.load_channels(factory=lambda d: cls.create_from_dict(d), url=url)
+        tmp = db.load_channels(factory=cls.create_from_dict, url=url)
         if len(tmp):
             return tmp[0]
         elif create:
-            tmp = podcastChannel(url)
+            tmp = PodcastChannel(url)
             if authentication_tokens is not None:
                 tmp.username = authentication_tokens[0]
                 tmp.password = authentication_tokens[1]
@@ -109,16 +138,21 @@ class podcastChannel(object):
             db.force_last_new(tmp)
             return tmp
 
-    @staticmethod
-    def create_from_dict(d):
-        c = podcastChannel()
-        for key in d:
-            if hasattr(c, key):
-                setattr(c, key, d[key])
-        return c
+    def episode_factory(self, d):
+        """
+        This function takes a dictionary containing key-value pairs for
+        episodes and returns a new PodcastEpisode object that is connected
+        to this PodcastChannel object.
+
+        Returns: A new PodcastEpisode object
+        """
+        return PodcastEpisode.create_from_dict(d, self)
 
     def update(self):
-        (updated, c) = self.fc.fetch(self.url, self)
+        (updated, c) = self.fc.fetch(self.url, self,
+                not gl.config.proxy_use_environment,
+                gl.config.http_proxy,
+                gl.config.ftp_proxy)
 
         if c is None:
             return ( False, None )
@@ -126,11 +160,6 @@ class podcastChannel(object):
         if c.status == 401:
             return ( False, 401 )
 
-        if c.status == 302 and hasattr(c, 'headers') and c.headers.get('content-type').startswith('text/html'):
-            log('Warning: Looks like a Wifi authentication page: %s', c.url, sender=self)
-            log('Acting as if the feed was not updated (FIXME!)', sender=self)
-            return (True, None)
-
         if self.url != c.url and c.status != 302:
             # The URL has changed, and the status code is not a temporary
             # redirect, so update the channel's URL accordingly for future use
@@ -183,13 +212,6 @@ class podcastChannel(object):
         # Marked as bulk because we commit after importing episodes.
         db.save_channel(self, bulk=True)
 
-        # Remove old episodes before adding the new ones.  This helps
-        # deal with hyperactive channels, such as TV news, when there
-        # can be more new episodes than the user wants in the list.
-        # By cleaning up old episodes before receiving the new ones we
-        # ensure that the user doesn't miss any.
-        db.purge(gl.config.max_episodes_per_feed, self.id)
-
         # Load all episodes to update them properly.
         existing = self.get_all_episodes()
 
@@ -201,7 +223,7 @@ class podcastChannel(object):
             episode = None
 
             try:
-                episode = podcastItem.from_feedparser_entry(entry, self)
+                episode = PodcastEpisode.from_feedparser_entry(entry, self)
             except Exception, e:
                 log('Cannot instantiate episode "%s": %s. Skipping.', entry.get('id', '(no id available)'), e, sender=self, traceback=True)
 
@@ -209,14 +231,19 @@ class podcastChannel(object):
                 self.count_new += 1
 
                 for ex in existing:
-                    if ex.guid == episode.guid:
-                        for k in ('title', 'title', 'description', 'link', 'pubDate'):
+                    if ex.guid == episode.guid or episode.is_duplicate(ex):
+                        for k in ('title', 'url', 'description', 'link', 'pubDate', 'guid'):
                             setattr(ex, k, getattr(episode, k))
                         self.count_new -= 1
                         episode = ex
 
                 episode.save(bulk=True)
 
+        # This *might* cause episodes to be skipped if there were more than
+        # max_episodes_per_feed items added to the feed between updates.
+        # The benefit is that it prevents old episodes from apearing as new
+        # in certain situations (see bug #340).
+        db.purge(gl.config.max_episodes_per_feed, self.id)
         db.commit()
         return ( True, None )
 
@@ -325,13 +352,29 @@ class podcastChannel(object):
             self.override_title = ''
 
     def get_downloaded_episodes(self):
-        return db.load_episodes(self, factory=lambda c: podcastItem.create_from_dict(c, self), state=db.STATE_DOWNLOADED)
+        return db.load_episodes(self, factory=self.episode_factory, state=db.STATE_DOWNLOADED)
     
-    def save_settings(self):
-        db.save_channel(self)
-    
-    def get_new_episodes( self):
-        return [episode for episode in db.load_episodes(self, factory=lambda x: podcastItem.create_from_dict(x, self)) if episode.state == db.STATE_NORMAL and not episode.is_played and not services.download_status_manager.is_download_in_progress(episode.url)]
+    def get_new_episodes(self, downloading=lambda e: False):
+        """
+        Get a list of new episodes. You can optionally specify
+        "downloading" as a callback that takes an episode as
+        a parameter and returns True if the episode is currently
+        being downloaded or False if not.
+
+        By default, "downloading" is implemented so that it
+        reports all episodes as not downloading.
+        """
+        def check_is_new(episode):
+            """
+            For a given episode, returns True if it is to
+            be considered new or False if it is "not new".
+            """
+            return episode.state == db.STATE_NORMAL and \
+                    not episode.is_played and \
+                    not downloading(episode)
+
+        return [episode for episode in db.load_episodes(self, \
+                factory=self.episode_factory) if check_is_new(episode)]
 
     def update_m3u_playlist(self):
         if gl.config.create_m3u_playlists:
@@ -373,22 +416,25 @@ class podcastChannel(object):
                 assert filename is not None
 
                 try:
-                    libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title, genre='Podcast')
+                    if (gl.config.rewrite_genre_tag):
+                        libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title, genre=gl.config.rewrite_genre_tag_with)
+                    else:
+                        libtagupdate.update_metadata_on_file(filename, title=item.title, artist=self.title, genre='Podcast')
                 except Exception, e:
                     log('Error while calling update_metadata_on_file(): %s', e)
 
             self.update_m3u_playlist()
 
     def get_all_episodes(self):
-        return db.load_episodes(self, factory = lambda d: podcastItem.create_from_dict(d, self))
+        return db.load_episodes(self, factory=self.episode_factory)
 
-    def iter_set_downloading_columns( self, model, iter, episode=None):
-        global ICON_AUDIO_FILE, ICON_VIDEO_FILE
+    def iter_set_downloading_columns(self, model, iter, episode=None, downloading=None):
+        global ICON_AUDIO_FILE, ICON_VIDEO_FILE, ICON_GENERIC_FILE
         global ICON_DOWNLOADING, ICON_DELETED, ICON_NEW
         
         if episode is None:
             url = model.get_value( iter, 0)
-            episode = db.load_episode(url, factory=lambda x: podcastItem.create_from_dict(x, self))
+            episode = db.load_episode(url, factory=self.episode_factory)
         else:
             url = episode.url
 
@@ -397,7 +443,7 @@ class podcastChannel(object):
         else:
             icon_size = 16
 
-        if services.download_status_manager.is_download_in_progress(url):
+        if downloading is not None and downloading(episode):
             status_icon = util.get_tree_icon(ICON_DOWNLOADING, icon_cache=self.icon_cache, icon_size=icon_size)
         else:
             if episode.state == db.STATE_NORMAL:
@@ -417,7 +463,7 @@ class podcastChannel(object):
                 elif file_type == 'video':
                     status_icon = util.get_tree_icon(ICON_VIDEO_FILE, not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
                 else:
-                    status_icon = util.get_tree_icon('unknown', not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
+                    status_icon = util.get_tree_icon(ICON_GENERIC_FILE, not episode.is_played, episode.is_locked, not episode.file_exists(), self.icon_cache, icon_size)
             elif episode.state == db.STATE_DELETED or episode.state == db.STATE_DOWNLOADED:
                 status_icon = util.get_tree_icon(ICON_DELETED, not episode.is_played, icon_cache=self.icon_cache, icon_size=icon_size)
             else:
@@ -426,13 +472,19 @@ class podcastChannel(object):
 
         model.set( iter, 4, status_icon)
 
-    def get_tree_model(self):
+    def get_tree_model(self, downloading=None):
         """
         Return a gtk.ListStore containing episodes for this channel
         """
-        new_model = gtk.ListStore( gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING, 
-            gobject.TYPE_BOOLEAN, gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, 
-            gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
+        DATA_TYPES = (str, str, str, bool, gtk.gdk.Pixbuf, str, str, str, str, str)
+
+        # TODO: Remove unused columns, make these symbolic names class
+        # members and use them everywhere, so we can change/reorder them
+        C_URL, C_TITLE, C_FILESIZE_TEXT, C_UNUSED0, C_STATUS_ICON, \
+                C_PUBLISHED_TEXT, C_DESCRIPTION, C_DESCRIPTION_STRIPPED, \
+                C_UNUSED1, C_EXTENSION = range(len(DATA_TYPES))
+
+        new_model = gtk.ListStore(*DATA_TYPES)
 
         log('Returning TreeModel for %s', self.url, sender = self)
         urls = []
@@ -447,14 +499,14 @@ class podcastChannel(object):
             new_iter = new_model.append((item.url, item.title, filelength, 
                 True, None, item.cute_pubdate(), description, util.remove_html_tags(item.description), 
                 'XXXXXXXXXXXXXUNUSEDXXXXXXXXXXXXXXXXXXX', item.extension()))
-            self.iter_set_downloading_columns( new_model, new_iter, episode=item)
+            self.iter_set_downloading_columns( new_model, new_iter, episode=item, downloading=downloading)
             urls.append(item.url)
         
         self.update_save_dir_size()
         return (new_model, urls)
     
     def find_episode( self, url):
-        return db.load_episode(url, factory=lambda x: podcastItem.create_from_dict(x, self))
+        return db.load_episode(url, factory=self.episode_factory)
 
     @classmethod
     def find_unique_folder_name(cls, foldername):
@@ -539,7 +591,7 @@ class podcastChannel(object):
     cover_file = property(fget=get_cover_file)
 
     def delete_episode_by_url(self, url):
-        episode = db.load_episode(url, lambda c: podcastItem.create_from_dict(c, self))
+        episode = db.load_episode(url, factory=self.episode_factory)
 
         if episode is not None:
             filename = episode.local_filename(create=False)
@@ -552,23 +604,27 @@ class podcastChannel(object):
         self.update_m3u_playlist()
 
 
-class podcastItem(object):
+class PodcastEpisode(PodcastModelObject):
     """holds data for one object in a channel"""
     MAX_FILENAME_LENGTH = 200
 
-    @staticmethod
-    def load(url, channel):
-        e = podcastItem(channel)
-        d = db.load_episode(url)
+    def reload_from_db(self):
+        """
+        Re-reads all episode details for this object from the
+        database and updates this object accordingly. Can be
+        used to refresh existing objects when the database has
+        been updated (e.g. the filename has been set after a
+        download where it was not set before the download)
+        """
+        d = db.load_episode(self.url)
         if d is not None:
-            for k, v in d.iteritems():
-                if hasattr(e, k):
-                    setattr(e, k, v)
-        return e
+            self.update_from_dict(d)
+
+        return self
 
     @staticmethod
     def from_feedparser_entry( entry, channel):
-        episode = podcastItem( channel)
+        episode = PodcastEpisode( channel)
 
         episode.title = entry.get( 'title', util.get_first_line( util.remove_html_tags( entry.get( 'summary', ''))))
         episode.link = entry.get( 'link', '')
@@ -701,14 +757,6 @@ class podcastItem(object):
             self.save()
             db.commit()
 
-    @staticmethod
-    def create_from_dict(d, channel):
-        e = podcastItem(channel)
-        for key in d:
-            if hasattr(e, key):
-                setattr(e, key, d[key])
-        return e
-
     @property
     def title_and_description(self):
         """
@@ -834,6 +882,11 @@ class podcastItem(object):
                 (episode_filename, extension_UNUSED) = util.filename_from_url(url)
                 fn_template = util.sanitize_filename(episode_filename, self.MAX_FILENAME_LENGTH)
 
+            # Use the video title for YouTube downloads
+            for yt_url in ('http://youtube.com/', 'http://www.youtube.com/'):
+                if self.url.startswith(yt_url):
+                    fn_template = self.title
+
             # If the basename is empty, use the md5 hexdigest of the URL
             if len(fn_template) == 0 or fn_template.startswith('redirect.'):
                 log('Report to bugs.gpodder.org: Podcast at %s with episode URL: %s', self.channel.url, self.url, sender=self)
@@ -972,6 +1025,11 @@ class podcastItem(object):
 
     played_prop = property(fget=get_played_string)
     
+    def is_duplicate( self, episode ):
+        if self.title == episode.title and self.pubDate == episode.pubDate:
+            log('Possible duplicate detected: %s', self.title)
+            return True
+        return False
 
 
 def update_channel_model_by_iter( model, iter, channel, color_dict,
@@ -1038,7 +1096,7 @@ def channels_to_model(channels, color_dict, cover_cache=None, max_width=0, max_h
 
 
 def load_channels():
-    return db.load_channels(lambda d: podcastChannel.create_from_dict(d))
+    return db.load_channels(factory=PodcastChannel.create_from_dict)
 
 def update_channels(callback_proc=None, callback_error=None, is_cancelled_cb=None):
     log('Updating channels....')
@@ -1082,7 +1140,7 @@ class LocalDBReader( object):
         return self.get_text( element.getElementsByTagName( name)[0].childNodes)
     
     def get_episode_from_element( self, channel, element):
-        episode = podcastItem( channel)
+        episode = PodcastEpisode(channel)
         episode.title = self.get_text_by_first_node( element, 'title')
         episode.description = self.get_text_by_first_node( element, 'description')
         episode.url = self.get_text_by_first_node( element, 'url')
@@ -1137,7 +1195,7 @@ class LocalDBReader( object):
         
         channel_element = rss.getElementsByTagName('channel')[0]
 
-        channel = podcastChannel( url = self.url)
+        channel = PodcastChannel(url=self.url)
         channel.title = self.get_text_by_first_node( channel_element, 'title')
         channel.description = self.get_text_by_first_node( channel_element, 'description')
         channel.link = self.get_text_by_first_node( channel_element, 'link')
